pub struct Channel {
pub channel_mask: Option<u32>,
pub sample_rate: Option<u32>,
pub sample_size: Option<u32>,
}Expand description
Audio channel configuration.
Fields
channel_mask: Option<u32>Audio channel mask. Set to 0 to disable the audio endpoint.
The audio channel mask is a bit mask of the audio channels. The mask is a 32-bit integer with each bit representing a channel. The least significant bit is channel 1. The mask is used to specify the audio channels that are present in the audio stream. For example, a stereo stream would have a mask of 0x3 (channel 1 and channel 2).
sample_rate: Option<u32>Audio sample rate (Hz)
sample_size: Option<u32>Audio sample size (bytes) so 2 bytes per sample (16 bit) would be 2.
Implementations