Uac1

Struct Uac1 

pub struct Uac1 { /* private fields */ }
USB Audio Class 1 (UAC1) function.

UAC1 is widely supported by hosts including car stereos, older Macs, game consoles, and embedded systems that may not support UAC2.

Implementations

Source

impl Uac1

pub fn builder() -> Uac1Builder

Creates a new USB Audio Class 1 (UAC1) builder with f_uac1 audio defaults.

pub fn new(capture: Channel, playback: Channel) -> (Uac1, Handle)

Creates a new USB Audio Class 1 (UAC1) function with the specified capture and playback channels.

pub fn status(&self) -> Status

Access to registration status.