#[non_exhaustive]pub struct Uac1Config {
pub channel: Channel,
pub mute_present: Option<bool>,
pub volume_present: Option<bool>,
pub volume_min: Option<i16>,
pub volume_max: Option<i16>,
pub volume_resolution: Option<i16>,
pub volume_name: Option<String>,
pub input_terminal_name: Option<String>,
pub input_terminal_channel_name: Option<String>,
pub output_terminal_name: Option<String>,
}Expand description
Audio device configuration for UAC1.
Fields are optional and will be set to f_uac1 default values if not specified, see
drivers/usb/gadget/function/u_uac1.h. Not all fields are supported by all kernels; permission
denied errors may occur if unsupported fields are set.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.channel: ChannelAudio channel configuration.
mute_present: Option<bool>If channel has mute
volume_present: Option<bool>If channel has volume
volume_min: Option<i16>Minimum volume (in 1/256 dB)
volume_max: Option<i16>Maximum volume (in 1/256 dB)
volume_resolution: Option<i16>Resolution of volume control (in 1/256 dB)
volume_name: Option<String>Name of the volume control function
input_terminal_name: Option<String>Name of the input terminal
input_terminal_channel_name: Option<String>Name of the input terminal channel
output_terminal_name: Option<String>Name of the output terminal