#[non_exhaustive]pub struct Lun {
pub read_only: bool,
pub cdrom: bool,
pub no_fua: bool,
pub removable: bool,
pub inquiry_string: String,
/* private fields */
}Expand description
Logical unit (LUN) of mass storage device (MSD).
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.read_only: boolFlag specifying access to the LUN shall be read-only.
This is implied if CD-ROM emulation is enabled as well as when it was impossible to open the backing file in R/W mode.
cdrom: boolFlag specifying that LUN shall be reported as being a CD-ROM.
no_fua: boolFlag specifying that FUA flag in SCSI WRITE(10,12).
removable: boolFlag specifying that LUN shall be indicated as being removable.
inquiry_string: StringInquiry string.
Implementations