Msd

Struct Msd 

pub struct Msd { /* private fields */ }
USB Mass Storage Device (MSD) function.

Implementations

Source

impl Msd

pub fn new(file: impl AsRef<Path>) -> Result<(Msd, Handle)>

Creates a new USB Mass Storage Device (MSD) with the specified backing file.

pub fn builder() -> MsdBuilder

Creates a new USB Mass Storage Device (MSD) builder.

pub fn status(&self) -> Status

Access to registration status.

pub fn force_eject(&self, lun: usize) -> Result<()>

Forcibly detach the backing file from the LUN, regardless of whether the host has allowed it.

pub fn set_file<P: AsRef<Path>>( &self, lun: usize, file: Option<P>, ) -> Result<()>

Set the path to the backing file for the LUN.