Utils for implementing USB gadget functions.
Structs
- Function
Dir - USB gadget function directory container.
- Status
- Provides access to the status of a USB function.
Enums
- State
- USB function registration state.
Traits
- Function
- USB gadget function.
Functions
- dev_
path - Find the device path in
/devfor the given major and minor numbers. - register_
remove_ handler - Register a function remove handler for the specified function driver.
- split_
function_ dir - Split configfs function directory path into driver name and instance name.