Udc

Struct Udc 

pub struct Udc { /* private fields */ }
USB device controller (UDC).

Call udcs to obtain the controllers available on the system.

Implementations

Source

impl Udc

pub fn name(&self) -> &OsStr

The name of the USB device controller.

pub fn a_alt_hnp_support(&self) -> Result<bool>

Indicates if an OTG A-Host supports HNP at an alternate port.

pub fn a_hnp_support(&self) -> Result<bool>

Indicates if an OTG A-Host supports HNP at this port.

pub fn b_hnp_enable(&self) -> Result<bool>

Indicates if an OTG A-Host enabled HNP support.

pub fn current_speed(&self) -> Result<Speed>

Indicates the current negotiated speed at this port.

None if unknown.

pub fn max_speed(&self) -> Result<Speed>

Indicates the maximum USB speed supported by this port.

pub fn is_a_peripheral(&self) -> Result<bool>

Indicates that this port is the default Host on an OTG session but HNP was used to switch roles.

pub fn is_otg(&self) -> Result<bool>

Indicates that this port support OTG.

pub fn state(&self) -> Result<UdcState>

Indicates current state of the USB Device Controller.

However not all USB Device Controllers support reporting all states.

pub fn start_srp(&self) -> Result<()>

Manually start Session Request Protocol (SRP).

pub fn set_soft_connect(&self, connect: bool) -> Result<()>

Connect or disconnect data pull-up resistors thus causing a logical connection to or disconnection from the USB host.

pub fn driver(&self) -> Result<OsString>

The kernel driver managing this USB device controller, e.g. dwc2, dwc3, dummy_udc, musb-hdrc, cdns3.

pub fn function(&self) -> Result<Option<OsString>>

Name of currently running USB Gadget Driver.