pub struct Uac1 { /* private fields */ }Expand description
USB Audio Class 1 (UAC1) function.
UAC1 is widely supported by hosts including car stereos, older Macs, game consoles, and embedded systems that may not support UAC2.
Implementations