Module util

Module util 

Utils for implementing USB gadget functions.

Structs

FunctionDir
USB gadget function directory container.
Status
Provides access to the status of a USB function.

Enums

State
USB function registration state.

Traits

Function
USB gadget function.

Functions

dev_path
Find the device path in /dev for the given major and minor numbers.
register_remove_handler
Register a function remove handler for the specified function driver.
split_function_dir
Split configfs function directory path into driver name and instance name.