pub struct Udc { /* private fields */ }Expand description
USB device controller (UDC).
Call udcs to obtain the controllers available on the system.
Implementations
Sourceimpl Udc
impl Udc
Sourcepub fn a_alt_hnp_support(&self) -> Result<bool>
pub fn a_alt_hnp_support(&self) -> Result<bool>
Indicates if an OTG A-Host supports HNP at an alternate port.
Sourcepub fn a_hnp_support(&self) -> Result<bool>
pub fn a_hnp_support(&self) -> Result<bool>
Indicates if an OTG A-Host supports HNP at this port.
Sourcepub fn b_hnp_enable(&self) -> Result<bool>
pub fn b_hnp_enable(&self) -> Result<bool>
Indicates if an OTG A-Host enabled HNP support.
Sourcepub fn current_speed(&self) -> Result<Speed>
pub fn current_speed(&self) -> Result<Speed>
Indicates the current negotiated speed at this port.
None if unknown.
Sourcepub fn max_speed(&self) -> Result<Speed>
pub fn max_speed(&self) -> Result<Speed>
Indicates the maximum USB speed supported by this port.
Sourcepub fn is_a_peripheral(&self) -> Result<bool>
pub fn is_a_peripheral(&self) -> Result<bool>
Indicates that this port is the default Host on an OTG session but HNP was used to switch roles.
Sourcepub fn state(&self) -> Result<UdcState>
pub fn state(&self) -> Result<UdcState>
Indicates current state of the USB Device Controller.
However not all USB Device Controllers support reporting all states.
Sourcepub fn set_soft_connect(&self, connect: bool) -> Result<()>
pub fn set_soft_connect(&self, connect: bool) -> Result<()>
Connect or disconnect data pull-up resistors thus causing a logical connection to or disconnection from the USB host.